// DisplayDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program01.h"
#include "DisplayDlg.h"

// Global flag
HANDLE	g_hMicrEvt = NULL;

// CDisplayDlg dialog

IMPLEMENT_DYNAMIC(CDisplayDlg, CDialog)

CDisplayDlg::CDisplayDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDisplayDlg::IDD, pParent)
{
	// Initialize the event.
	g_hMicrEvt = CreateEvent(NULL, FALSE, FALSE ,"ReadEvent");

	m_nHandle				 = 0;
	m_isInitialized			 = FALSE;
	m_needCloseMonPrinter	 = FALSE;
	m_needMICRCancelReadBack = FALSE;
}

CDisplayDlg::~CDisplayDlg()
{
	// Close the handle
	if (g_hMicrEvt != NULL) {
		CloseHandle(g_hMicrEvt);
	}

	EndOperation();
}

void CDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CDisplayDlg, CDialog)
#ifdef _WIN64
	ON_BN_CLICKED(IDC_BUTTON_READ, CDisplayDlg::OnBnClickedButtonRead)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE, CDisplayDlg::OnBnClickedButtonRemove)
#else
	ON_BN_CLICKED(IDC_BUTTON_READ, &CDisplayDlg::OnBnClickedButtonRead)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE, &CDisplayDlg::OnBnClickedButtonRemove)
#endif
END_MESSAGE_MAP()


// CDisplayDlg message handlers
void CDisplayDlg::OnBnClickedButtonRead()
{
	if (StartOperation()) {
		ReadMICR();
	}
	else {
		EndOperation();
	}
}

// Eject check if "Remove" button is clicked.
void CDisplayDlg::OnBnClickedButtonRemove()
{
	BYTE byMicrStat = 0;

	if (StartOperation()) {
		if (m_statAPI.BiMICRGetStatus(m_nHandle, &byMicrStat) != SUCCESS) {
			AfxMessageBox(_T("Failed to get status."));
		}

		/*
		 * Check if the MICR unit is selected and eject the check
		 */
		if (!(byMicrStat & BITMASK_MICRSELECT)) {
			m_statAPI.BiMICREjectCheck(m_nHandle);
		}
	}
}

/*****************************************************************************
  M I C R    C H E C K    R E A D I N G                                      *
 *****************************************************************************/
BOOL CDisplayDlg::ReadMICR()
{
	BOOL bRet = TRUE;

	ResetEvent(g_hMicrEvt);

	memset(m_readCharBuff, 0, sizeof(m_readCharBuff));

	// Start check reading.
	if (m_statAPI.BiMICRReadCheck(m_nHandle, MICR_FONT_E13B, 0) != SUCCESS) {
		return FALSE;
	}
	
	// Wait for the event.
	WaitForSingleObject(g_hMicrEvt, INFINITE);

	// Error check.
	if (m_status & BIT_ERR_READERROR) {
		bRet = FALSE;
	}

	// Display the string data.
	SetDlgItemText(IDC_EDIT_DATA, (LPCSTR)m_readCharBuff);

	if (bRet) {
		AfxMessageBox(_T("MICR reading complete."), MB_ICONINFORMATION);
	}
	else {
		AfxMessageBox(_T("Error occurred during MICR reading."));
	}

	return bRet;
}

BOOL CDisplayDlg::StartOperation()
{
	BOOL bRet = TRUE;

	if (m_isInitialized) {
		return bRet;
	}

	// Initialize the Status API object. This loads the functions from EpsStmApi.dll.
	if (m_statAPI.Initialize() == FALSE) {
		AfxMessageBox(_T("Failed to open StatusAPI."));
		bRet = FALSE;
	}

	// Open a printer status monitor for the selected printer.
	if (bRet) {
		m_nHandle = m_statAPI.BiOpenMonPrinter(TYPE_PRINTER, EPS_PRINTER_NAME);

		if (m_nHandle < 0) {
			AfxMessageBox(_T("Failed to open printer status monitor."));
			bRet = FALSE;
		}
		else {
			m_needCloseMonPrinter = TRUE;
		}
	}
	
	// Declare the variables to store data read from MICR.
	if (bRet) {

		BYTE typeA = 0;
		BYTE typeB = 0;
		BYTE font  = 0;
		BYTE exrom = 0;

		// Get the type of printer and confirm if it supports MICR reading and if it is equipped with an MICR unit.
		if (m_statAPI.BiGetType(m_nHandle, &typeA, &font, &exrom, &typeB) == SUCCESS) {
			if (!(typeA & BITMASK_WITHMICR)) {
				AfxMessageBox(_T("MICR is not supported with this model."), MB_ICONINFORMATION);
				bRet = FALSE;
			}
			else if (!(typeB & BITMASK_MICREQUIPPED)) {
				AfxMessageBox(_T("MICR is not installed."), MB_ICONINFORMATION);
				bRet = FALSE;
			}
		}
		else {
			bRet = FALSE;
		}
	}

	 // Set the callback function.
	 // The callback function is executed when MICR reading is complete.
	if (bRet) {
		m_readBuffSize = sizeof(m_readCharBuff);
		m_status = 0;
		m_detail = 0;
		
		memset(m_readCharBuff, 0, sizeof(m_readCharBuff));

		if (m_statAPI.BiMICRSetReadBackFunction(m_nHandle,
												CBMICRRead,
												&m_readBuffSize,
												m_readCharBuff,
												&m_status,
												&m_detail) != SUCCESS ) {
			AfxMessageBox(_T("Failed to set callback function."));
			bRet = FALSE;
		}
		else {
			m_needMICRCancelReadBack = TRUE;
		}
	}

	// Select the data handling. See the manual for the valid parameters.
	// For this example, the following parameters are used
	//		charSelect		: 1 (Replace unknown characters with '?')
	//		detailSelect	: 0 (No detailed information)
	//		errorSelect		: 1 (Continue operation even if character error occurs)
	if (bRet) {
		if (m_statAPI.BiMICRSelectDataHandling(m_nHandle, 1, 0, 1) != SUCCESS) {
			bRet = FALSE;
		}
	}

	if (bRet) {
		m_isInitialized = TRUE;
	}
	else {
		AfxMessageBox(_T("Error occurred during initialization."));
	}

	return bRet;
}

void CDisplayDlg::EndOperation()
{
	if (m_needMICRCancelReadBack) {
		if (m_statAPI.BiMICRCancelReadBack(m_nHandle) != SUCCESS) {
			AfxMessageBox(_T("Failed to cancel read callback."));
		}
		else {
			m_needMICRCancelReadBack = FALSE;
		}
	}

	 // Always close the Status Monitor after using the Status API.
	if (m_needCloseMonPrinter) {
		if (m_statAPI.BiCloseMonPrinter(m_nHandle) != SUCCESS) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		else {
			m_nHandle = 0;
			m_isInitialized = FALSE;
			m_needCloseMonPrinter = FALSE;
		}
	}
}

// The callback function that will monitor MICR scanning status.
int CALLBACK EXPORT CBMICRRead(void)
{
	SetEvent(g_hMicrEvt);
	return SUCCESS;
}
