// DisplayDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program05.h"
#include "DisplayDlg.h"

// Event handles for synchronization of StatusAPI
HANDLE g_hMicrEvt = NULL;
HANDLE g_hPrinterEvt = NULL;
DWORD g_dwStatus = 0;

// CDisplayDlg dialog

IMPLEMENT_DYNAMIC(CDisplayDlg, CDialog)

CDisplayDlg::CDisplayDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDisplayDlg::IDD, pParent)
{
	 // Initialize the events to synchronize each step.
	g_hMicrEvt = CreateEvent(NULL, FALSE, FALSE, "MICREvent");
	g_hPrinterEvt = CreateEvent(NULL, FALSE, FALSE, "PrinterEvent");

	m_nHandle				 = -1;
	m_isInitialized			 = FALSE;
	m_needCloseMonPrinter	 = FALSE;
	m_needCancelStatusBack	 = FALSE;
	m_needMICRCancelReadBack = FALSE;
}

CDisplayDlg::~CDisplayDlg()
{
	 // Close the handles to all events.
	if (NULL != g_hMicrEvt) {
		CloseHandle(g_hMicrEvt);
	}

	if (NULL != g_hPrinterEvt) {
		CloseHandle(g_hPrinterEvt);
	}

	EndOperation();
}

void CDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CDisplayDlg, CDialog)
#ifdef _WIN64
	ON_BN_CLICKED(IDC_BUTTON_READ, CDisplayDlg::OnBnClickedButtonRead)
#else
	ON_BN_CLICKED(IDC_BUTTON_READ, &CDisplayDlg::OnBnClickedButtonRead)
#endif
	ON_WM_PAINT()
END_MESSAGE_MAP()


// CDisplayDlg message handlers

// The executed function when the Start button is clicked.
// The sequence of the execution is:
// MICR scanning => Print endorse => Print slip
void CDisplayDlg::OnBnClickedButtonRead()
{
	if (StartOperation()) {
		if (ReadMICR()) {
			if (PrintEndorse()) {
				PrintSlip();
			}
		}
	}
	else {
		EndOperation();
	}
}

/*****************************************************************************
 * M I C R    C H E C K    R E A D I N G                                     *
 *****************************************************************************/
BOOL CDisplayDlg::ReadMICR()
{
	BOOL bRet = TRUE;

	ResetEvent(g_hMicrEvt);

	memset(m_readCharBuff, 0, sizeof(m_readCharBuff));

	// Start check reading.
	if (m_statAPI.BiMICRReadCheck(m_nHandle, MICR_FONT_E13B, 0) != SUCCESS) {
		return FALSE;
	}
	
	// Wait for the event.
	WaitForSingleObject(g_hMicrEvt, INFINITE);

	// Error check.
	if (m_status & BIT_ERR_READERROR) {
		bRet = FALSE;
	}
	else {		
		// Reload the check.
		if (m_statAPI.BiMICRLoadCheck(m_nHandle) != SUCCESS) {
			bRet = FALSE;
		}
	}

	// Display the string data.
	SetDlgItemText(IDC_EDIT_DATA, (LPCSTR)m_readCharBuff);

	if (bRet) {
		AfxMessageBox(_T("MICR reading complete."), MB_ICONINFORMATION);
	}
	else {
		AfxMessageBox(_T("Error occurred during MICR reading."));
		m_statAPI.BiMICREjectCheck(m_nHandle);
	}

	return bRet;
}

/***************************************************************************
 * E N D O R S E    P R I N T I N G                                        *
 ***************************************************************************/
BOOL CDisplayDlg::PrintEndorse()
{
	BOOL bRet = TRUE;

	CDC dc;

	// Use the endorse driver to endorsement printing.
    if (dc.CreateDC(NULL, EPS_ENDORSEPRINTER_NAME, NULL, NULL)) {
		CFont	font;
		CFont	*old;
		DOCINFO di = {0};     // must have DOCINFO for CDCStartDoc(DOCINFO)
		DWORD dwPrintStatus = 0;
		int x = 30;
		int y = 10;

		di.cbSize = sizeof(DOCINFO);
		di.lpszDocName = "Endorse Test";

		dc.StartDoc(&di);
		dc.StartPage();

		font.CreatePointFont(90, "FontA11", &dc);
		old = dc.SelectObject(&font);

		dc.TextOut(x, y, "Endorse printing.."); y += LINE_OFFSET;

		dc.EndPage();
		dc.EndDoc();

		// Wait until callback function will say that the task is done.
		WaitForSingleObject(g_hPrinterEvt, INFINITE);

		// Display the status/error message.
		dwPrintStatus = g_dwStatus;
		DisplayStatusMessage(dwPrintStatus);

		// Restore the previously used font.
		dc.SelectObject(old);
		font.DeleteObject();
		dc.DeleteDC();
	}
	else {
		AfxMessageBox(_T("Printer is not available."));
		m_statAPI.BiMICREjectCheck(m_nHandle);
		bRet = FALSE;
    }

	return bRet;
}

/**************************************************************************
 * S L I P    P R I N T I N G                                             *
 **************************************************************************/
BOOL CDisplayDlg::PrintSlip()
{
	BOOL bRet = TRUE;

	CDC dc;

	ResetEvent(g_hPrinterEvt);

	// Use the slip driver to front printing.
	if(dc.CreateDC(NULL, EPS_SLIPPRINTER_NAME, NULL, NULL)) {
		CFont	font;
		CFont	*old;
		DOCINFO di = {0};     // must have DOCINFO for CDCStartDoc(DOCINFO)
		DWORD dwPrintStatus = 0;
		int x = 480;
		int y = 180;

		di.cbSize = sizeof(DOCINFO);
		di.lpszDocName = "Slip Test";

		dc.StartDoc(&di);
		dc.StartPage();

		font.CreatePointFont(110, "Consolas", &dc);
		old = dc.SelectObject(&font);

		dc.TextOut(x, y, "Slip printing.."); y += LINE_OFFSET;

		dc.SelectObject(old);
		font.DeleteObject();

		dc.EndPage();
		dc.EndDoc();

		// Wait until callback function will say that the task is done.
		WaitForSingleObject(g_hPrinterEvt, INFINITE);
		
		// Display the status/error message.
		dwPrintStatus = g_dwStatus;
		DisplayStatusMessage(dwPrintStatus);
		
		// Restore the previously used font.
		dc.SelectObject(old);
		font.DeleteObject();
		dc.DeleteDC();
	}
	else {
		AfxMessageBox(_T("Printer is not available."));
		m_statAPI.BiMICREjectCheck(m_nHandle);
		bRet = FALSE;
    }

	return bRet;
}

void CDisplayDlg::DisplayStatusMessage(DWORD dwPrintStatus)
{
	if (dwPrintStatus & ASB_PRINT_SUCCESS) {
		AfxMessageBox(_T("Printing complete."), MB_ICONINFORMATION);
	}
	else if (dwPrintStatus & ASB_COVER_OPEN) {
		AfxMessageBox(_T("Cover is open."));
		m_statAPI.BiCancelError(m_nHandle);
	}
	
	else if (dwPrintStatus & ASB_AUTOCUTTER_ERR) {
		AfxMessageBox(_T("Autocutter error occurred."));
		m_statAPI.BiCancelError(m_nHandle);
	}
	
	else if (dwPrintStatus & ASB_RECEIPT_NEAR_END) {
		AfxMessageBox(_T("Roll paper end sensor: paper not present."));
		m_statAPI.BiCancelError(m_nHandle);
	}
}

BOOL CDisplayDlg::StartOperation()
{
	BOOL bRet = TRUE;

	if (m_isInitialized) {
		return bRet;
	}

	// Initialize the Status API object. This loads the functions from EpsStmApi.dll.
	if (m_statAPI.Initialize() == FALSE) {
		AfxMessageBox(_T("Failed to open StatusAPI."));
		bRet = FALSE;
	}

	// Open a printer status monitor for the selected printer.
	if (bRet) {
		m_nHandle = m_statAPI.BiOpenMonPrinter(TYPE_PRINTER, EPS_SLIPPRINTER_NAME);

		if (m_nHandle < 0) {
			AfxMessageBox(_T("Failed to open printer status monitor."));
			bRet = FALSE;
		}
		else {
			m_needCloseMonPrinter = TRUE;
		}
	}

	// Declare the variables to store data read from MICR.
	if (bRet) {
		BYTE typeA = 0;
		BYTE typeB = 0;
		BYTE font  = 0;
		BYTE exrom = 0;

		// Get the type of printer and confirm if it supports MICR reading and if it is equipped with an MICR unit.
		if (m_statAPI.BiGetType(m_nHandle, &typeA, &font, &exrom, &typeB) == SUCCESS) {
			if (!(typeA & BITMASK_WITHMICR)) {
				AfxMessageBox(_T("MICR is not supported with this model."), MB_ICONINFORMATION);
				bRet = FALSE;
			}
			else if (!(typeB & BITMASK_MICREQUIPPED)) {
				AfxMessageBox(_T("MICR is not installed."), MB_ICONINFORMATION);
				bRet = FALSE;
			}
		}
		else {
			bRet = FALSE;
		}
	}

	// Set the read callback functions for print.
	if  (bRet) {
		if (m_statAPI.BiSetStatusBackFunction(m_nHandle, CBPrinterStatus) != SUCCESS) {
			AfxMessageBox(_T("Failed to set callback function."));
			bRet = FALSE;
		}
		else {
			m_needCancelStatusBack = TRUE;
		}
	}

	 // Set the callback function.
	 // The callback function is executed when MICR reading is complete.
	if (bRet) {
		m_readBuffSize = sizeof(m_readCharBuff);
		m_status = 0;
		m_detail = 0;
		
		memset(m_readCharBuff, 0, sizeof(m_readCharBuff));

		if (m_statAPI.BiMICRSetReadBackFunction(m_nHandle,
												CBMICRRead,
												&m_readBuffSize,
												m_readCharBuff,
												&m_status,
												&m_detail) != SUCCESS) {
			AfxMessageBox(_T("Failed to set callback function."));
			bRet = FALSE;
		}
		else {
			m_needMICRCancelReadBack = TRUE;
		}
	}

	// Select the data handling. See the manual for the valid parameters.
	// For this example, the following parameters are used
	//		charSelect		: 1 (Replace unknown characters with '?')
	//		detailSelect	: 0 (No detailed information)
	//		errorSelect		: 1 (Continue operation even if character error occurs)
	if (bRet) {
		if (m_statAPI.BiMICRSelectDataHandling(m_nHandle, 1, 0, 1) != SUCCESS) {
			bRet = FALSE;
		}
	}

	if (bRet) {
		m_isInitialized = TRUE;
	}
	else {
		AfxMessageBox(_T("Error occurred during initialization."));
	}

	return bRet;
}

void CDisplayDlg::EndOperation()
{
	if (m_needMICRCancelReadBack) {
		if (m_statAPI.BiMICRCancelReadBack(m_nHandle) != SUCCESS) {
			AfxMessageBox(_T("Failed to cancel read callback."));
		}
		else {
			m_needMICRCancelReadBack = FALSE;
		}
	}

	if (m_needCancelStatusBack) {
		if (m_statAPI.BiCancelStatusBack(m_nHandle) != SUCCESS) {
			AfxMessageBox(_T("Failed to cancel read callback."));
		}
		else {
			m_needCancelStatusBack = FALSE;
		}
	}

	 // Always close the Status Monitor after using the Status API.
	if (m_needCloseMonPrinter) {
		if (m_statAPI.BiCloseMonPrinter(m_nHandle) != SUCCESS) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		else {
			m_nHandle = -1;
			m_isInitialized = FALSE;
			m_needCloseMonPrinter = FALSE;
		}
	}
}

// The callback function that will monitor printer/printing status.
int CALLBACK CBPrinterStatus(DWORD dwStatus)
{
	if (dwStatus & ASB_NO_RESPONSE	  || 
		dwStatus & ASB_PRINT_SUCCESS  ||
		dwStatus & ASB_AUTOCUTTER_ERR ||
		dwStatus & ASB_COVER_OPEN	  ||
		dwStatus & ASB_RECEIPT_END) {
		g_dwStatus = dwStatus;
		SetEvent(g_hPrinterEvt);
	}
	return SUCCESS;
}

// The callback function that will monitor MICR scanning status.
int CALLBACK EXPORT CBMICRRead(void)
{
	SetEvent(g_hMicrEvt);
	return SUCCESS;
}
