// PrintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program04.h"
#include "PrintDlg.h"
#include ".\printdlg.h"


// CPrintDlg dialog

IMPLEMENT_DYNAMIC(CPrintDlg, CDialog)
CPrintDlg::CPrintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintDlg::IDD, pParent)
{
}

CPrintDlg::~CPrintDlg()
{
}

void CPrintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPrintDlg, CDialog)
	ON_BN_CLICKED(IDC_PRINT_BUTTON, OnBnClickedPrintButton)
#ifdef _WIN64
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, CPrintDlg::OnBnClickedCloseButton)
#else
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, &CPrintDlg::OnBnClickedCloseButton)
#endif
END_MESSAGE_MAP()


// CPrintDlg message handlers

void CPrintDlg::OnBnClickedPrintButton()
{
	Print();
}

void CPrintDlg::Print() {
    DOCINFO di;     // must have DOCINFO for CDCStartDoc(DOCINFO)
    CFont	*old, font;

	/*
	 * Clean the document information structure
	 */
    memset(&di, 0, sizeof(DOCINFO));
    di.cbSize = sizeof(DOCINFO);
    di.lpszDocName = "Testing";
	
	CDC			dc;
	/*
	 * Create a device context for printing
	 */
	if(! dc.CreateDC(NULL, EPS_PRINTER_NAME, NULL, NULL))
    {
		AfxMessageBox(_T("Printer is not available."));
		return;
    }
	
	dc.StartDoc(&di);

	LPBITMAPINFO info;
	HBITMAP      hbit;
	BITMAP       bm;
	int          nColors  = 0;
	int          sizeinfo = 0;
	RGBQUAD      rgb[256];

	/*
	 * The following line loads the bitmap from a file
	 */
	hbit = (HBITMAP) LoadImage(0,
							   LOGO_FILENAME,
							   IMAGE_BITMAP,
							   0,
							   0,
							   LR_CREATEDIBSECTION | LR_LOADFROMFILE);
	if (hbit == NULL ) {
		dc.AbortDoc();
		dc.DeleteDC();
		return;
	}

	GetObject(hbit, sizeof(BITMAP), (LPVOID) &bm);

	nColors = (1 << bm.bmBitsPixel);
	if(nColors > 256)
	  nColors=0;           

	sizeinfo = sizeof(BITMAPINFO) + (nColors * sizeof(RGBQUAD));   // This is the size required

	info = (LPBITMAPINFO) malloc(sizeinfo);                        // Storing info in memory

	info->bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
	info->bmiHeader.biWidth         = bm.bmWidth;
	info->bmiHeader.biHeight        = bm.bmHeight;
	info->bmiHeader.biPlanes        = 1;
	info->bmiHeader.biBitCount      = bm.bmBitsPixel * bm.bmPlanes;
	info->bmiHeader.biCompression   = BI_RGB;
	info->bmiHeader.biSizeImage     = bm.bmWidthBytes * bm.bmHeight;
	info->bmiHeader.biXPelsPerMeter = 0;
	info->bmiHeader.biYPelsPerMeter = 0;
	info->bmiHeader.biClrUsed       = 0;
	info->bmiHeader.biClrImportant  = 0;

	if(nColors <= 256)
	{
	  HBITMAP hOldBitmap;
	  HDC     hMemDC     = CreateCompatibleDC(NULL);      // Creating an auxiliary device context

	  hOldBitmap = (HBITMAP) SelectObject(hMemDC, hbit);  // Select 'hbit' in 'it'
	  GetDIBColorTable(hMemDC, 0, nColors, rgb);          // Obtaining the color table information

	  /*
	   * Now we pass this color information to "info" structure
	   */
	  for(int iCnt = 0; iCnt < nColors; ++iCnt)
	  {
		info->bmiColors[iCnt].rgbRed   = rgb[iCnt].rgbRed;
		info->bmiColors[iCnt].rgbGreen = rgb[iCnt].rgbGreen;
		info->bmiColors[iCnt].rgbBlue  = rgb[iCnt].rgbBlue;
	  }
	      
	  SelectObject(hMemDC, hOldBitmap);
	  DeleteDC(hMemDC);
	}

	/*
	 * Draw the bitmap
	 */
	StretchDIBits(dc.GetSafeHdc(), 
				200, 0,
				bm.bmWidth*2,
				bm.bmHeight*2, 
				0, 0, 
				bm.bmWidth,
				bm.bmHeight, 
				bm.bmBits, 
				info, 
				DIB_RGB_COLORS, SRCCOPY);

	DeleteObject(hbit);
	free(info);

	LPTEXTMETRIC	lpMetrics = new TEXTMETRIC[1];
	LONG			lineOffset;

	/*
	 * Create and select the device font for the receipt
	 */
	font.CreatePointFont(95, "FontA11", &dc);
	old = dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	int x = 30, y = 80;

	/*
	 * Print the receipt text
	 */
	dc.TextOut(x, y, "123xxstreet,xxxcity,xxxxstate"); y += lineOffset;
	dc.TextOut(x, y, "        TEL   9999-99-9999       C#2");	y+= lineOffset;
	dc.TextOut(x, y, "    November.23, 2007     PM 4:24"); y+= (lineOffset*2);
	dc.TextOut(x, y, "apples                       $20.00"); y+= lineOffset;
	dc.TextOut(x, y, "grapes                       $30.00"); y+= lineOffset;
	dc.TextOut(x, y, "bananas                      $40.00"); y+= lineOffset;
	dc.TextOut(x, y, "lemons                       $50.00"); y+= lineOffset;
	dc.TextOut(x, y, "oranges                      $60.00"); y+= (lineOffset*2);
	dc.TextOut(x, y, "Tax excluded.               $200.00"); y+= lineOffset;
	dc.TextOut(x, y, "Tax     5.0%                 $10.00"); y+= lineOffset;
	dc.TextOut(x, y, "___________________________________"); 
	
	dc.SelectObject(old);
	font.DeleteObject();
	font.CreatePointFont(190, "FontA22", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;
	y+= lineOffset;
	dc.TextOut(x, y, "Total     $210.00"); y+= lineOffset;
	dc.SelectObject(old);
	font.DeleteObject();

	font.CreatePointFont(95, "FontA11", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	dc.TextOut(x, y, "Customer's payment         $250.00"); y+= lineOffset;
	dc.TextOut(x, y, "Change                      $40.00"); y+= (lineOffset*2);
	
	/*
	 * Create space below the receipt
	 */
	dc.TextOut(x, y, "       ");

	/*
	 * Restore the previously used font
	 */
	dc.SelectObject(old);
	font.DeleteObject();
	delete[] lpMetrics;

	dc.EndPage();
    dc.EndDoc();
    dc.DeleteDC();

}


void CPrintDlg::OnBnClickedCloseButton()
{
	OnOK();
}
