// PrintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program06.h"
#include "StatusAPI.h"
#include "EpsStmApi.h"
#include "PrintDlg.h"
#include ".\printdlg.h"


// CPrintDlg dialog

IMPLEMENT_DYNAMIC(CPrintDlg, CDialog)
CPrintDlg::CPrintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintDlg::IDD, pParent)
{
}

CPrintDlg::~CPrintDlg()
{
}

void CPrintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPrintDlg, CDialog)
	ON_BN_CLICKED(IDC_PRINT_BUTTON, OnBnClickedPrintButton)
#ifdef _WIN64
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, CPrintDlg::OnBnClickedCloseButton)
#else
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, &CPrintDlg::OnBnClickedCloseButton)
#endif
END_MESSAGE_MAP()


// CPrintDlg message handlers

void CPrintDlg::OnBnClickedPrintButton()
{
	Print();
}

void CPrintDlg::Print() {
    DOCINFO di;     // must have DOCINFO for CDCStartDoc(DOCINFO)

    memset(&di, 0, sizeof(DOCINFO)); // make a clean start
    di.cbSize = sizeof(DOCINFO);
    di.lpszDocName = "Testing";
	
	int	nHandle ;

	/*
	 * Initialize the Status API object. This loads the functions from EpsStmApi.dll
	 */
	if ( m_statAPI.Initialize() == FALSE ) {
		AfxMessageBox(_T("Failed to open StatusAPI."));
		return;
	}

	/*
	 * Open a printer status monitor for the selected printer
	 */
	nHandle = m_statAPI.BiOpenMonPrinter(TYPE_PRINTER, EPS_PRINTER_NAME) ;
	if ( nHandle <= 0 )
	{
		AfxMessageBox(_T("Failed to open printer status monitor."));
		return;
	}
	
	/*
	 * Create the device context for printing
	 */
	CDC			dc;
	if(! dc.CreateDC(NULL, EPS_PRINTER_NAME, NULL, NULL))
    {
		if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		AfxMessageBox(_T("Printer is not available."));
		return;
    }
	
	dc.StartDoc(&di);

	LPBITMAPINFO info;
	HBITMAP      hbit;
	BITMAP       bm;
	int          nColors  = 0;
	int          sizeinfo = 0;
	RGBQUAD      rgb[256];

	/*
	 * The following line loads the bitmap from a file
	 */
	hbit = (HBITMAP) LoadImage(0,
							   LOGO_FILENAME,
							   IMAGE_BITMAP,
							   0,
							   0,
							   LR_CREATEDIBSECTION | LR_LOADFROMFILE);
	if (hbit == NULL ) {
		dc.AbortDoc();
		dc.DeleteDC();
		if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		return;
	}

	GetObject(hbit, sizeof(BITMAP), (LPVOID) &bm);

	nColors = (1 << bm.bmBitsPixel);
	if(nColors > 256)
	  nColors=0;           

	sizeinfo = sizeof(BITMAPINFO) + (nColors * sizeof(RGBQUAD));   // This is the size required

	info = (LPBITMAPINFO) malloc(sizeinfo);                        // Storing info in memory

	info->bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
	info->bmiHeader.biWidth         = bm.bmWidth;
	info->bmiHeader.biHeight        = bm.bmHeight;
	info->bmiHeader.biPlanes        = 1;
	info->bmiHeader.biBitCount      = bm.bmBitsPixel * bm.bmPlanes;
	info->bmiHeader.biCompression   = BI_RGB;
	info->bmiHeader.biSizeImage     = bm.bmWidthBytes * bm.bmHeight;
	info->bmiHeader.biXPelsPerMeter = 0;
	info->bmiHeader.biYPelsPerMeter = 0;
	info->bmiHeader.biClrUsed       = 0;
	info->bmiHeader.biClrImportant  = 0;

	if(nColors <= 256)
	{
		HBITMAP hOldBitmap;
		HDC     hMemDC     = CreateCompatibleDC(NULL);      // Creating an auxiliary device context

		hOldBitmap = (HBITMAP) SelectObject(hMemDC, hbit);  // Select 'hbit' in 'it'
		GetDIBColorTable(hMemDC, 0, nColors, rgb);          // Obtaining the color table information


		/*
		* Now we pass this color information to "info" structure
		*/
		for(int iCnt = 0; iCnt < nColors; ++iCnt)
		{
			info->bmiColors[iCnt].rgbRed   = rgb[iCnt].rgbRed;
			info->bmiColors[iCnt].rgbGreen = rgb[iCnt].rgbGreen;
			info->bmiColors[iCnt].rgbBlue  = rgb[iCnt].rgbBlue;
		}
			
		SelectObject(hMemDC, hOldBitmap);
		DeleteDC(hMemDC);
	}

	/*
	 * Draw the bitmap
	 */
	StretchDIBits(dc.GetSafeHdc(), 
				200, 0,
				bm.bmWidth*2,
				bm.bmHeight*2, 
				0, 0, 
				bm.bmWidth,
				bm.bmHeight, 
				bm.bmBits, 
				info, 
				DIB_RGB_COLORS, SRCCOPY);

	DeleteObject(hbit);
	free(info);

	LPTEXTMETRIC	lpMetrics = new TEXTMETRIC[1];
	LONG			lineOffset;

	/*
	 * Create and select the device font for printing the receipt
	 */
    CFont	font;
	CFont	*old;
	font.CreatePointFont(95, "FontA11", &dc);
	
	old = dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	int x = 30, y = 80;

	/*
	 * Print the receipt text
	 */
	dc.TextOut(x, y, "123xxstreet,xxxcity,xxxxstate"); y += lineOffset;
	dc.TextOut(x, y, "        TEL   9999-99-9999       C#2");	y+= lineOffset;
	dc.TextOut(x, y, "    November.23, 2007     PM 4:24"); y+= (lineOffset*2);
	dc.TextOut(x, y, "apples                       $20.00"); y+= lineOffset;
	dc.TextOut(x, y, "grapes                       $30.00"); y+= lineOffset;
	dc.TextOut(x, y, "bananas                      $40.00"); y+= lineOffset;
	dc.TextOut(x, y, "lemons                       $50.00"); y+= lineOffset;
	dc.TextOut(x, y, "oranges                      $60.00"); y+= (lineOffset*2);
	dc.TextOut(x, y, "Tax excluded.               $200.00"); y+= lineOffset;
	dc.TextOut(x, y, "Tax     5.0%                 $10.00"); y+= lineOffset;
	dc.TextOut(x, y, "___________________________________"); 
	
	dc.SelectObject(old);
	font.DeleteObject();
	font.CreatePointFont(190, "FontA22", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;
	y+= lineOffset;
	dc.TextOut(x, y, "Total     $210.00"); y+= lineOffset;
	dc.SelectObject(old);
	font.DeleteObject();

	font.CreatePointFont(95, "FontA11", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	dc.TextOut(x, y, "Customer's payment         $250.00"); y+= lineOffset;
	dc.TextOut(x, y, "Change                      $40.00"); y+= (lineOffset*2);
	
	/*
	 * Create space below the receipt
	 */
	dc.TextOut(x, y, "       ");

	/*
	 * Restore the previously used font
	 */
	dc.SelectObject(old);
	font.DeleteObject();
	delete[] lpMetrics;

    dc.EndPage();
    dc.EndDoc();

	DWORD	dwPrnStatus ;

	/*
	 * Use the Status API's BiGetStatus function to continuously check for the printer's status
	 */
	do {
		m_statAPI.BiGetStatus(nHandle, &(dwPrnStatus));
	} while (! ( dwPrnStatus & ASB_PRINT_SUCCESS ) ); 
	/* Perform the status check as long as the status is not ASB_PRINT_SUCCESS.
	 * The status retrieved should be checked using a Bitwise AND operation, since each
	 * status is represented as a single or multiple bits. */ 


	/* Notify printing completion. */
	AfxMessageBox(_T("Printing complete."), MB_ICONINFORMATION );
	
	/*
	 * Always close the Status Monitor after using the Status API
	 */
	if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
		AfxMessageBox(_T("Failed to close printer status monitor."));
		return;
	}

    dc.DeleteDC();
}
void CPrintDlg::OnBnClickedCloseButton()
{
	OnOK();
}
