// PrintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program11.h"
#include "StatusAPI.h"
#include "EpsStmApi.h"
#include "PrintDlg.h"
#include ".\printdlg.h"
#include "winspool.h"

HANDLE	hAsync;
DWORD	g_dwStatus;

// CPrintDlg dialog

IMPLEMENT_DYNAMIC(CPrintDlg, CDialog)
CPrintDlg::CPrintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintDlg::IDD, pParent)
{
	hAsync = CreateEvent(NULL, FALSE, FALSE, "StatusEvent");
}

CPrintDlg::~CPrintDlg()
{
	CloseHandle(hAsync);
}

void CPrintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPrintDlg, CDialog)
	ON_BN_CLICKED(IDC_PRINT_BUTTON, OnBnClickedPrintButton)
#ifdef _WIN64
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, CPrintDlg::OnBnClickedCloseButton)
#else
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, &CPrintDlg::OnBnClickedCloseButton)
#endif
END_MESSAGE_MAP()


// CPrintDlg message handlers

void CPrintDlg::OnBnClickedPrintButton()
{
	Print();
}

void CPrintDlg::Print() {

	DWORD	dwPrintStatus;
    DOCINFO di;     // must have DOCINFO for CDCStartDoc(DOCINFO)

    /*
	 * Clean the document information structure
	 */
    memset(&di, 0, sizeof(DOCINFO));
    di.cbSize = sizeof(DOCINFO);
    di.lpszDocName = "Testing";
	
	/*
	 * Initialize the Status API object. This loads the functions from EpsStmApi.dll
	 */
	int	nHandle ;	// The handle for the printer monitor
	if ( m_statAPI.Initialize() == FALSE ) {
		AfxMessageBox(_T("Failed to open StatusAPI."));
		return;
	}

	/*
	 * Open a printer status monitor for the selected printer
	 */
	nHandle = m_statAPI.BiOpenMonPrinter(TYPE_PRINTER, EPS_PRINTER_NAME) ;
	if ( nHandle <= 0 )
	{
		AfxMessageBox(_T("Failed to open printer status monitor."));
		return;
	}
	
	/*
	 * This feature of the Status API allows the program to register a callback function.
	 * The call back function is executed whenever there is a change in the status.
	 */
	if ( m_statAPI.BiSetStatusBackFunction(nHandle, CBGetStatus) != SUCCESS )
	{
		AfxMessageBox(_T("Failed to set callback function."));
		if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		return;
	}
	
	/*
	 * Create a device context for printing
	 */
    CDC			dc;
	if(! dc.CreateDC(NULL, EPS_PRINTER_NAME, NULL, NULL))
    {
		AfxMessageBox(_T("Printer is not available."));
		if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		return;
    }
	
	// Begin printing
	int		nJobId;
	nJobId =	dc.StartDoc(&di);

	LPBITMAPINFO info;
	HBITMAP      hbit;
	BITMAP       bm;
	int          nColors  = 0;
	int          sizeinfo = 0;
	RGBQUAD      rgb[256];

	/*
	 * The following line loads the bitmap from a file
	 */
	hbit = (HBITMAP) LoadImage(0,
							   LOGO_FILENAME,
							   IMAGE_BITMAP,
							   0,
							   0,
							   LR_CREATEDIBSECTION | LR_LOADFROMFILE);
	if (hbit == NULL ) {
		dc.AbortDoc();
		dc.DeleteDC();
		if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
			AfxMessageBox(_T("Failed to close printer status monitor."));
		}
		return;
	}
	GetObject(hbit, sizeof(BITMAP), (LPVOID) &bm);

	nColors = (1 << bm.bmBitsPixel);
	if(nColors > 256)
	  nColors=0;           

	sizeinfo = sizeof(BITMAPINFO) + (nColors * sizeof(RGBQUAD));   // This is the size required

	info = (LPBITMAPINFO) malloc(sizeinfo);                        // Storing info in memory

	info->bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
	info->bmiHeader.biWidth         = bm.bmWidth;
	info->bmiHeader.biHeight        = bm.bmHeight;
	info->bmiHeader.biPlanes        = 1;
	info->bmiHeader.biBitCount      = bm.bmBitsPixel * bm.bmPlanes;
	info->bmiHeader.biCompression   = BI_RGB;
	info->bmiHeader.biSizeImage     = bm.bmWidthBytes * bm.bmHeight;
	info->bmiHeader.biXPelsPerMeter = 0;
	info->bmiHeader.biYPelsPerMeter = 0;
	info->bmiHeader.biClrUsed       = 0;
	info->bmiHeader.biClrImportant  = 0;

	if(nColors <= 256)
	{
		HBITMAP hOldBitmap;
		HDC     hMemDC     = CreateCompatibleDC(NULL);      // Creating an auxiliary device context

		hOldBitmap = (HBITMAP) SelectObject(hMemDC, hbit);  // Select 'hbit' in 'it'
		GetDIBColorTable(hMemDC, 0, nColors, rgb);          // Obtaining the color table information

		// Now we pass this color information to "info"
		for(int iCnt = 0; iCnt < nColors; ++iCnt)
		{
			info->bmiColors[iCnt].rgbRed   = rgb[iCnt].rgbRed;
			info->bmiColors[iCnt].rgbGreen = rgb[iCnt].rgbGreen;
			info->bmiColors[iCnt].rgbBlue  = rgb[iCnt].rgbBlue;
		}
			
		SelectObject(hMemDC, hOldBitmap);
		DeleteDC(hMemDC);
	}

	/*
	 * Draw the bitmap
	 */
	StretchDIBits(dc.GetSafeHdc(), 
				200, 0,
				bm.bmWidth*2,
				bm.bmHeight*2, 
				0, 0, 
				bm.bmWidth,
				bm.bmHeight, 
				bm.bmBits, 
				info, 
				DIB_RGB_COLORS, SRCCOPY);

	DeleteObject(hbit);
	free(info);

	LPTEXTMETRIC	lpMetrics = new TEXTMETRIC[1];
	LONG			lineOffset;

    CFont	font;
	CFont	*old;
	/* 
	 * Create and select the device font for printing
	 */
	font.CreateFont(-25, 0, 0, 0, 
					FW_NORMAL, FALSE, FALSE, 0,  
					ANSI_CHARSET, 
					OUT_DEFAULT_PRECIS, 
					CLIP_DEFAULT_PRECIS, 
					DEFAULT_QUALITY, 
					DEFAULT_PITCH | FF_SWISS, 
					"FontA11");
	
	old = dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	int x = 30, y = 80;

	/*
	 * Print the receipt text
	 */
	dc.TextOut(x, y, "123xxstreet,xxxcity,xxxxstate"); y += lineOffset;
	dc.TextOut(x, y, "        TEL   9999-99-9999       C#2");	y+= lineOffset;
	dc.TextOut(x, y, "    November.23, 2007     PM 4:24"); y+= (lineOffset*2);
	dc.TextOut(x, y, "apples                       $20.00"); y+= lineOffset;
	dc.TextOut(x, y, "grapes                       $30.00"); y+= lineOffset;
	dc.TextOut(x, y, "bananas                      $40.00"); y+= lineOffset;
	dc.TextOut(x, y, "lemons                       $50.00"); y+= lineOffset;
	dc.TextOut(x, y, "oranges                      $60.00"); y+= (lineOffset*2);
	dc.TextOut(x, y, "Tax excluded.               $200.00"); y+= lineOffset;
	dc.TextOut(x, y, "Tax     5.0%                 $10.00"); y+= lineOffset;
	dc.TextOut(x, y, "___________________________________"); 
	
	dc.SelectObject(old);
	font.DeleteObject();
	font.CreatePointFont(190, "FontA22", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;
	y+= lineOffset;
	dc.TextOut(x, y, "Total     $210.00"); y+= lineOffset;
	dc.SelectObject(old);
	font.DeleteObject();

	font.CreatePointFont(95, "FontA11", &dc);
	dc.SelectObject(&font);
	dc.GetTextMetrics(lpMetrics);
	lineOffset = lpMetrics->tmHeight;

	dc.TextOut(x, y, "Customer's payment         $250.00"); y+= lineOffset;
	dc.TextOut(x, y, "Change                      $40.00"); y+= (lineOffset*2);
	
	/*
	 * Create space below the receipt
	 */
	dc.TextOut(x, y, "       ");

	/*
	 * Restore the previously used font
	 */
	dc.SelectObject(old);
	font.DeleteObject();
	delete[] lpMetrics;

    dc.EndPage();
    dc.EndDoc();
	dc.DeleteDC();

	/*
	 * Check the global flags continuously until any one is set to TRUE
	 */
	WaitForSingleObject(hAsync, INFINITE);
	dwPrintStatus	= g_dwStatus;
	m_statAPI.BiCancelStatusBack(nHandle);
	
	if ( dwPrintStatus & ASB_PRINT_SUCCESS ) {
		/*
		* Notify print completion
		*/
		AfxMessageBox(_T("Printing complete."), MB_ICONINFORMATION);
	}
	else {
		/*
		* Notify any errors that occur.
		*/
		if ( dwPrintStatus & ASB_PAPER_END || dwPrintStatus & ASB_RECEIPT_END ) {
			AfxMessageBox(_T("Roll paper end sensor: paper not present."));
			m_statAPI.BiCancelError(nHandle);
			// Cancel the print job
			HANDLE		hPrinter ;
			if ( !OpenPrinter(EPS_PRINTER_NAME, &hPrinter, NULL) ) {
				AfxMessageBox(_T("Failed to open printer."));
			}
			else {
				if ( SetJob ( hPrinter, nJobId, 0, NULL, JOB_CONTROL_DELETE)) {
					AfxMessageBox(_T("Print job cancelled."));
				}
				else
				{
					AfxMessageBox(_T("Failed to cancel print job."));
				}
				
				ClosePrinter(hPrinter);
			}
		}
		else {
			if ( dwPrintStatus & ASB_NO_RESPONSE ) {
				AfxMessageBox(_T("No response."));
				m_statAPI.BiCancelError(nHandle);
			}

			else if ( dwPrintStatus & ASB_COVER_OPEN ) {
				AfxMessageBox(_T("Cover is open."));
				m_statAPI.BiCancelError(nHandle);
			}
			
			else if ( dwPrintStatus & ASB_AUTOCUTTER_ERR ) {
				AfxMessageBox(_T("Autocutter error occurred."));
				m_statAPI.BiCancelError(nHandle);
			}
		}
	}

	/*
	 * Always close the Status Monitor after using the Status API
	 */
	if ( m_statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
		AfxMessageBox(_T("Failed to close printer status monitor."));
	}
   
}
void CPrintDlg::OnBnClickedCloseButton()
{
	OnOK();
}

int CALLBACK CBGetStatus(DWORD dwStatus)
{
	g_dwStatus	= dwStatus;
	
	if ( dwStatus & ASB_NO_RESPONSE || dwStatus & ASB_PRINT_SUCCESS ||
		dwStatus & ASB_AUTOCUTTER_ERR || dwStatus & ASB_COVER_OPEN ||
		dwStatus & ASB_PAPER_END || dwStatus & ASB_RECEIPT_END ) {
			SetEvent(hAsync);
		}

	return SUCCESS;
}
