// PrintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Program14.h"
#include "PrintDlg.h"
#include ".\printdlg.h"

#include <process.h> // For the printing thread

#include "EpsStmApi.h"
#include "StatusAPI.h"

// Global flag
BOOL	bCancelPressed;

// CPrintDlg dialog

IMPLEMENT_DYNAMIC(CPrintDlg, CDialog)
CPrintDlg::CPrintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintDlg::IDD, pParent)
{
	hThread = NULL;
}

CPrintDlg::~CPrintDlg()
{
	// Make sure that the thread handle is freed
	if ( hThread != NULL ) {
		CloseHandle (hThread);
	}
}

void CPrintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPrintDlg, CDialog)
	ON_BN_CLICKED(IDC_PRINT_BUTTON, OnBnClickedPrintButton)
	ON_BN_CLICKED(IDC_CANCEL_BUTTON, OnBnClickedCancelButton)
	ON_BN_CLICKED(IDC_BUTTON_CLOSE, OnBnClickedButtonClose)
END_MESSAGE_MAP()


// CPrintDlg message handlers

void CPrintDlg::OnBnClickedPrintButton()
{
	/*
	 * For this program, instead of using the Print() function, we use
	 * an asynchronous thread so that the user can cancel the "waiting" print job
	 */
	hThread = (HANDLE)_beginthreadex(NULL, 100, &printDocument, this, 0, &ThreadId);
}

void CPrintDlg::OnBnClickedCancelButton() {
	bCancelPressed = TRUE;
}

/*
 * This thread waits for printer to become available and then
 * prints the document.
 */
unsigned __stdcall printDocument ( void *arglist ) {
	BOOL bContinue = TRUE, bOpen = FALSE, bLocked = FALSE;
	DOCINFO di;     // must have DOCINFO for CDCStartDoc(DOCINFO)
	
	CPrintDlg	*cprint	= (CPrintDlg*)arglist;
    
	/*
	 * Clean the document information structure
	 */
    memset(&di, 0, sizeof(DOCINFO));
    di.cbSize = sizeof(DOCINFO);
    di.lpszDocName = "Testing";

	int	nHandle;
	
	/*
	 * Initialize the Status API object. This loads the functions from EpsStmApi.dll
	 */
	CStatusAPI statAPI;
	if ( ! statAPI.Initialize() ) {
		AfxMessageBox(_T("Failed to open StatusAPI."));
		bContinue = FALSE;
	}

	if ( bContinue )
	{
		/*
		* Initialize the global flag to signal that CANCEL has been pressed.
		*/
		bCancelPressed	= FALSE;
		
		/*
		* Display the "Wait" dialog.
		*/
		nHandle = statAPI.BiOpenMonPrinter(TYPE_PRINTER, EPS_PRINTER_NAME);

		/*
		* Open a printer status monitor for the selected printer
		*/
		if ( nHandle < 0 ) {
			AfxMessageBox(_T("Failed to open printer status monitor."));
			bContinue = FALSE;
		}
		else
		{
			bOpen = TRUE;
		}
	}

	if ( bContinue )
	{
		cprint->GetDlgItem(IDC_STATIC_WAIT)->ShowWindow(SW_SHOW);
		cprint->GetDlgItem(IDC_CANCEL_BUTTON)->ShowWindow(SW_SHOW);
		/*
		* Try to get a lock on the printer.
		* If this fails, it means the printer is locked by another process.
		* Continue attempting to lock the printer until it is successful or until the user presses Cancel
		*/
		while ( statAPI.BiLockPrinter (nHandle, 3000) < 0 ) {
			if  ( bCancelPressed ) {
				break;
			}
		}

		/*
		* If the Cancel button was pressed, end the thread.
		*/
		if ( bCancelPressed ) {
			AfxMessageBox(_T("Print job cancelled."), MB_ICONINFORMATION );
			bContinue = FALSE;
		}
		else
		{
			bLocked = TRUE;
		}
	}

	/*
	 * Hide the WAIT window and the Cancel button
	 */
	cprint->GetDlgItem(IDC_STATIC_WAIT)->ShowWindow(SW_HIDE);
	cprint->GetDlgItem(IDC_CANCEL_BUTTON)->ShowWindow(SW_HIDE);

	/*
	 * Create a device context for printing
	 */
    CDC dc;
	if ( bContinue )
	{
		if(! dc.CreateDC(NULL, EPS_PRINTER_NAME, NULL, NULL) )
		{
			AfxMessageBox(_T("Printer is not available."));
			if ( statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
				AfxMessageBox(_T("Failed to close printer status monitor."));
			}
			bContinue = FALSE;
		}
	}

	/* Print the text */
	if ( bContinue )
	{
		dc.StartDoc(&di);
		dc.TextOut(20, 10, "Hello APD!");

		dc.EndPage();
		dc.EndDoc();
		dc.DeleteDC();
	}

	/*
	 * Unlock the printer so that other processes can use it.
	 */
	if ( bLocked && statAPI.BiUnlockPrinter ( nHandle ) != SUCCESS ) {
		AfxMessageBox(_T("Failed to unlock printer."));
		bContinue = FALSE;
	}

	/*
	 * Always close the Status Monitor after using the Status API
	 */
	if ( bOpen && statAPI.BiCloseMonPrinter(nHandle) != SUCCESS ) {
		AfxMessageBox(_T("Failed to close printer status monitor."));
	}

	_endthreadex(0);
	return 0;
}
void CPrintDlg::OnBnClickedButtonClose()
{
	OnOK();
}
