Imports System.Drawing.Printing
Imports System.IO
Imports com.epson.pos.driver

Public Class Form1

    ' Constant variables holding the printer names.
    Private Const PRINTER_NAME_SLIP = "EPSON TM-H6000VI Slip"
    Private Const PRINTER_NAME_ENDORSE = "EPSON TM-H6000VI Endorse"

    ' Bit masks for Type ID acquired through GetType.
    Private Const BITMASK_MICREQUIPPED = 8
    Private Const BITMASK_WITHMICR = 8

    ' Variables/Objects.
    Private m_objAPI As New StatusAPI

    Private WithEvents pdPrintEndorse As PrintDocument
    Private WithEvents pdPrintSlip As PrintDocument
    Private printFont As Font

    Dim isPrintFinish As Boolean
    Dim isReadFinish As Boolean
    Dim printStatus As ASB

    Dim MICRdata As String
    Dim MICRstatus As MICRStatus
    Dim MICRdetail As MICRDetail

    Dim isInitialize As Boolean = False
    Dim needCloseMonPrinter As Boolean = False
    Dim needCancelStatusBack As Boolean = False
    Dim needMICRCancelReadBack As Boolean = False

    ' The executed function when the Start button is clicked.
    ' The sequence of the execution is:
    ' MICR scanning => Print endorse => Print slip
    Private Sub cmdStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStart.Click
        If StartOperation() Then
            If ReadMICR() Then
                If PrintEndorse() Then
                    PrintSlip()
                End If
            End If
        Else
            EndOperation()
        End If
    End Sub

    ' The executed function when the Exit button is clicked.
    Private Sub cmdClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click
        EndOperation()
        Close()
    End Sub

    ' ***************************************************************************
    ' * M I C R    C H E C K    R E A D I N G                                   *
    ' ***************************************************************************
    Private Function ReadMICR() As Boolean

        Dim bRet As Boolean = True

        isReadFinish = False

        ' Executes the check reading.
        If Not m_objAPI.MICRReadCheck(MicrFont.MICR_FONT_E13B, 0) = ErrorCode.SUCCESS Then
            Return False
        End If

        ' Wait until callback function will say that the task is done.
        While Not isReadFinish
            System.Threading.Thread.Sleep(500)
        End While

        ' Error check
        If Not (MICRstatus And MICRstatus.ABNORMAL_COMPLETION) = 0 Then
            bRet = False
        Else
            ' Reload the check.
            If Not m_objAPI.MICRLoadCheck() = ErrorCode.SUCCESS Then
                bRet = False
            End If
        End If

        ' Display the string data.
        txtData.Text = MICRdata

        If bRet Then
            MessageBox.Show("Reading complete.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Information)
        Else
            MessageBox.Show("Error occurred during MICR reading.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Exclamation)
            m_objAPI.MICREjectCheck()
        End If

        Return bRet

    End Function

    ' ***************************************************************************
    ' * E N D O R S E    P R I N T I N G                                        *
    ' ***************************************************************************
    Private Function PrintEndorse() As Boolean

        Dim bRet As Boolean = True
        pdPrintEndorse = New PrintDocument

        isPrintFinish = False

        ' Use the endorse driver to endorsement printing
        pdPrintEndorse.PrinterSettings.PrinterName = PRINTER_NAME_ENDORSE

        If pdPrintEndorse.PrinterSettings.IsValid Then

            printFont = New Font("Consolas", 11, FontStyle.Regular, GraphicsUnit.Point) 'Substituted to FontA Font

            pdPrintEndorse.DocumentName = "Endorse test"
            ' Start printing
            pdPrintEndorse.Print()

            ' Wait until callback function will say that the task is done.
            Do
                System.Threading.Thread.Sleep(500)
            Loop While Not isPrintFinish

            ' Display the status/error message.
            DisplayStatusMessage()
        Else

            MessageBox.Show("Printer is not available.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Exclamation)
            m_objAPI.MICREjectCheck()
            bRet = False
        End If

        Return bRet

    End Function

    ' ***************************************************************************
    ' * S L I P    P R I N T I N G                                              *
    ' ***************************************************************************
    Private Function PrintSlip() As Boolean

        Dim bRet As Boolean = True
        pdPrintSlip = New PrintDocument

        isPrintFinish = False

        ' Use the slip driver to front printing
        pdPrintSlip.PrinterSettings.PrinterName = PRINTER_NAME_SLIP

        If pdPrintSlip.PrinterSettings.IsValid Then

            printFont = New Font("Consolas", 11, FontStyle.Regular, GraphicsUnit.Point) 'Substituted to FontA Font

            pdPrintSlip.DocumentName = "Slip test"
            ' Start printing.
            pdPrintSlip.Print()

            ' Wait until callback function will say that the task is done.
            Do
                System.Threading.Thread.Sleep(500)
            Loop While Not isPrintFinish

            ' Display the status/error message.
            DisplayStatusMessage()
        Else

            MessageBox.Show("Printer is not available.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Exclamation)
            m_objAPI.MICREjectCheck()
            bRet = False
        End If

        Return bRet

    End Function

    Private Sub DisplayStatusMessage()

        If (printStatus And ASB.ASB_PRINT_SUCCESS) = ASB.ASB_PRINT_SUCCESS Then
            MessageBox.Show("Printing complete.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Information)

        ElseIf (printStatus And ASB.ASB_NO_RESPONSE) = ASB.ASB_NO_RESPONSE Then
            MessageBox.Show("No response.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        ElseIf (printStatus And ASB.ASB_COVER_OPEN) = ASB.ASB_COVER_OPEN Then
            MessageBox.Show("Cover is open.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        ElseIf (printStatus And ASB.ASB_AUTOCUTTER_ERR) = ASB.ASB_AUTOCUTTER_ERR Then
            MessageBox.Show("Autocutter error occurred.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        ElseIf (printStatus And ASB.ASB_RECEIPT_END) = ASB.ASB_RECEIPT_END Then
            MessageBox.Show("Roll paper end sensor: paper not present.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        End If
    End Sub

    ' The event handler function when pdPrintEndorse.Print is called.
    ' This is where the actual printing of sample data to the Endorse printer is made.
    Private Sub pdPrintEndorse_Print(ByVal sender As System.Object, ByVal e As PrintPageEventArgs) Handles pdPrintEndorse.PrintPage
        e.Graphics.PageUnit = GraphicsUnit.Point
        e.Graphics.DrawString("Endorse printing..", printFont, Brushes.Black, 15, 10)
        e.HasMorePages = False
    End Sub

    ' The event handler function when pdPrintSlip.Print is called.
    ' This is where the actual printing of sample data to the Slip printer is made.
    Private Sub pdPrintSlip_Print(ByVal sender As System.Object, ByVal e As PrintPageEventArgs) Handles pdPrintSlip.PrintPage
        e.Graphics.PageUnit = GraphicsUnit.Point
        e.Graphics.DrawString("Slip printing..", printFont, Brushes.Black, 480, 80)
        e.HasMorePages = False
    End Sub

    Private Function StartOperation() As Boolean
        Dim bRet As Boolean = True

        If isInitialize Then
            Return bRet
        End If

        ' Open a printer status monitor for the selected printer
        ' Either name of Endorse or Slip printers can be supplied in the parameter for printer name.
        If Not m_objAPI.OpenMonPrinter(OpenType.TYPE_PRINTER, PRINTER_NAME_SLIP) = ErrorCode.SUCCESS Then
            MessageBox.Show("Failed to open printer status monitor.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Exclamation)
            bRet = False
        Else
            needCloseMonPrinter = True
        End If

        ' Declare the variables to store data read from MICR
        If bRet Then
            Dim typeA As Byte
            Dim typeB As Byte
            Dim font As Byte
            Dim exrom As Byte

            If m_objAPI.GetType(typeA, font, exrom, typeB) = ErrorCode.SUCCESS Then

                If (typeA And BITMASK_WITHMICR) = 0 Then
                    MessageBox.Show("MICR is not supported with this model.", _
                                    "Program05", _
                                    MessageBoxButtons.OK, _
                                    MessageBoxIcon.Exclamation)
                    bRet = False
                End If

                If (typeB And BITMASK_MICREQUIPPED) = 0 Then
                    MessageBox.Show("MICR is not installed.", _
                                    "Program05", _
                                    MessageBoxButtons.OK, _
                                    MessageBoxIcon.Exclamation)
                    bRet = False
                End If
            End If
        End If

        ' Set the read callback functions for print.
        If bRet Then
            ' Associate the created callback function to the event handler of StatusAPI.
            AddHandler m_objAPI.StatusCallback, AddressOf CBPrinterStatus

            If Not m_objAPI.SetStatusBack() = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to set callback function.", _
                                "Program05", _
                                MessageBoxButtons.OK, _
                                MessageBoxIcon.Exclamation)
                bRet = False
            Else
                needCancelStatusBack = True
            End If
        End If

        ' Set the read callback functions for MICR.
        ' The callback function is executed when MICR reading is complete.
        If bRet Then
            ' Associate the created callback function to the event handler of StatusAPI.
            AddHandler m_objAPI.MICRReadCallback, AddressOf CBMICRRead

            If Not m_objAPI.MICRSetReadBack() = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to set callback function.", _
                                "Program05", _
                                MessageBoxButtons.OK, _
                                MessageBoxIcon.Exclamation)
                bRet = False
            Else
                needMICRCancelReadBack = True
            End If
        End If

        ' Set the data handling for the MICR scanner
        '		charSel		: MF_CONTINUE_ANALYSIS (Replace with ?)
        '		detailSel	: MF_DETAILINFO_NOTADDED (No detailed information)
        '		errorSel	: ES_CONTINUE_ALL (Continue operation even with error)
        If bRet Then
            If Not m_objAPI.MICRSelectDataHandling(CharSelect.MF_CONTINUE_ANALYSIS, DetailSelect.MF_DETAILINFO_NOTADDED, ErrorSelect.ES_CONTINUE_ALL) = ErrorCode.SUCCESS Then
                bRet = False
            End If
        End If

        If bRet Then
            isInitialize = True
        Else
            MessageBox.Show("Error occurred during initialization.", _
                            "Program05", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Exclamation)
        End If

        Return bRet

    End Function

    Private Sub EndOperation()

        If needMICRCancelReadBack Then

            If Not m_objAPI.MICRCancelReadBack() = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to cancel read callback.", _
                                "Program05", _
                                MessageBoxButtons.OK, _
                                MessageBoxIcon.Exclamation)
            Else
                needMICRCancelReadBack = False
            End If
        End If

        If needCancelStatusBack Then
            If Not m_objAPI.CancelStatusBack() = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to cancel read callback.", _
                                "Program05", _
                                MessageBoxButtons.OK, _
                                MessageBoxIcon.Exclamation)
            Else
                needCancelStatusBack = False
            End If
        End If

        ' Always close the Status Monitor after using the Status API
        If needCloseMonPrinter Then
            If Not m_objAPI.CloseMonPrinter() = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to close printer status monitor.", _
                                "Program05", _
                                MessageBoxButtons.OK, _
                                MessageBoxIcon.Exclamation)
            Else
                isInitialize = False
                needCloseMonPrinter = False
            End If
        End If
    End Sub

    ' The callback function that will monitor printer/printing status.
    Public Sub CBPrinterStatus(ByVal dwStatus As ASB)

        If (dwStatus And ASB.ASB_NO_RESPONSE) = ASB.ASB_NO_RESPONSE Or _
                (dwStatus And ASB.ASB_PRINT_SUCCESS) = ASB.ASB_PRINT_SUCCESS Or _
                (dwStatus And ASB.ASB_COVER_OPEN) = ASB.ASB_COVER_OPEN Or _
                (dwStatus And ASB.ASB_AUTOCUTTER_ERR) = ASB.ASB_AUTOCUTTER_ERR Or _
                (dwStatus And ASB.ASB_RECEIPT_END) = ASB.ASB_RECEIPT_END Then
            printStatus = dwStatus
            isPrintFinish = True

        End If

    End Sub

    ' The callback function that will monitor MICR scanning status.
    Public Sub CBMICRRead(ByVal data As String, ByVal status As MICRStatus, ByVal detail As MICRDetail)

        MICRdata = data
        MICRstatus = status
        MICRdetail = detail

        isReadFinish = True

    End Sub

End Class
