Imports System.Drawing.Printing
Imports com.epson.pos.driver

Public Class Form1

    ' Constant variable holding the Printer name.
    Private Const PRINTER_NAME = "EPSON TM-T88V Receipt"

    ' Variables/Objects.
    Dim m_objAPI As New StatusAPI
    Private WithEvents pdPrint As PrintDocument

    ' The executed function when the Print button is clicked.
    Private Sub cmdPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPrint.Click
        Dim isfinish As Boolean

        Try
            ' Open a printer status monitor for the selected printer.
            If m_objAPI.OpenMonPrinter(OpenType.TYPE_PRINTER, PRINTER_NAME) = ErrorCode.SUCCESS Then

                pdPrint = New PrintDocument

                ' Change the printer to the indicated printer.
                pdPrint.PrinterSettings.PrinterName = PRINTER_NAME

                If pdPrint.PrinterSettings.IsValid Then

                    pdPrint.DocumentName = "Testing"
                    ' Start printing
                    pdPrint.Print()

                    ' Check printing status.
                    isfinish = False

                    ' Perform the status check as long as the status is not ASB_PRINT_SUCCESS.
                    Do
                        If m_objAPI.Status.ToString.Contains(ASB.ASB_PRINT_SUCCESS.ToString) Then
                            isfinish = True
                        End If
                    Loop While Not isfinish

                    ' Notify printing completion.
                    MessageBox.Show("Printing complete.", "Program06", MessageBoxButtons.OK, MessageBoxIcon.Information)

                Else
                    MessageBox.Show("Printer is not available.", "Program06", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                End If

                ' Always close the Status Monitor after using the Status API.
                If m_objAPI.CloseMonPrinter() <> ErrorCode.SUCCESS Then
                    MessageBox.Show("Failed to close printer status monitor.", "Program06", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                End If

            Else
                MessageBox.Show("Failed to open printer status monitor.", "Program06", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If

        Catch ex As Exception
            MessageBox.Show("Failed to open StatusAPI.", "Program06", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ' The event handler function when pdPrint.Print is called.
    ' This is where the actual printing of sample data to the printer is made.
    Private Sub pdPrint_Print(ByVal sender As System.Object, ByVal e As PrintPageEventArgs) Handles pdPrint.PrintPage
        Dim x, y, lineOffset As Integer

        ' Instantiate font objects used in printing.
        Dim printFont As New Font("Lucida Console", 8, FontStyle.Regular, GraphicsUnit.Point) 'Substituted to FontA Font

        e.Graphics.PageUnit = GraphicsUnit.Point

        ' Draw the bitmap
        x = 79
        y = 0
        e.Graphics.DrawImage(pbImage.Image, x, y, pbImage.Image.Width - 13, pbImage.Image.Height - 10)

        ' Print the receipt text
        lineOffset = printFont.GetHeight(e.Graphics) - 4
        x = 10
        y = 25 + lineOffset
        e.Graphics.DrawString("123xxstreet,xxxcity,xxxxstate", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("        TEL   9999-99-9999       C#2", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("    November.23, 2007     PM 4:24", printFont, Brushes.Black, x, y)
        y += (lineOffset * 5)
        e.Graphics.DrawString("apples                       $20.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("grapes                       $30.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("bananas                      $40.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("lemons                       $50.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("oranges                      $60.00", printFont, Brushes.Black, x, y)
        y += (lineOffset * 5)
        e.Graphics.DrawString("Tax excluded.               $200.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("Tax     5.0%                 $10.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("___________________________________", printFont, Brushes.Black, x, y)

        printFont = New Font("MingLiU", 19, FontStyle.Regular, GraphicsUnit.Point)
        lineOffset = printFont.GetHeight(e.Graphics) - 4
        y += lineOffset
        e.Graphics.DrawString("Total     $210.00", printFont, Brushes.Black, x - 1, y)

        printFont = New Font("Lucida Console", 8, FontStyle.Regular, GraphicsUnit.Point)
        lineOffset = printFont.GetHeight(e.Graphics)
        y += lineOffset
        e.Graphics.DrawString("Customer's payment         $250.00", printFont, Brushes.Black, x, y)
        y += lineOffset
        e.Graphics.DrawString("Change                      $40.00", printFont, Brushes.Black, x, y - 2)

        ' Indicate that no more data to print, and the Print Document can now send the print data to the spooler.
        e.HasMorePages = False
    End Sub

    ' The executed function when the Close button is clicked.
    Private Sub cmdClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click
        Close()
    End Sub
End Class
