Imports System.Drawing.Printing
Imports com.epson.pos.driver

Public Class Form1

    ' Constant variable holding the Printer name.
    Private Const PRINTER_NAME = "EPSON TM-T88V Receipt"

    ' Variables/Objects.
    Private m_objAPI As StatusAPI
    Private WithEvents pdPrint As PrintDocument

    ' The event handler function when the Program executes.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        m_objAPI = New StatusAPI

        lblMessage.Visible = False
        cmdCancel.Visible = False
        tmrTimer.Interval = 200

    End Sub

    ' The executed function when the Print button is clicked.
    Private Sub cmdPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPrint.Click
        pdPrint = New PrintDocument
        pdPrint.PrinterSettings.PrinterName = PRINTER_NAME

        Try
            ' Open a printer status monitor for the selected printer.
            If m_objAPI.OpenMonPrinter(OpenType.TYPE_PRINTER, PRINTER_NAME) = ErrorCode.SUCCESS Then

                ' Start the Timer.
                ' The timer is used here to imitate threading.
                ' Threading is used here so that other functions will still be accessible while printing is being executed.
                tmrTimer.Start()

            Else
                MessageBox.Show("Failed to open printer status monitor.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If

        Catch ex As Exception
            MessageBox.Show("Failed to open StatusAPI.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

    End Sub

    ' The event handler function when pdPrint.Print is called.
    ' This is where the actual printing of sample data to the printer is made.
    Private Sub pdPrint_Print(ByVal sender As System.Object, ByVal e As PrintPageEventArgs) Handles pdPrint.PrintPage
        Dim printFont As New Font("Lucida Console", 8, FontStyle.Regular, GraphicsUnit.Point) 'Substituted to FontA Font

        e.Graphics.PageUnit = GraphicsUnit.Point

        e.Graphics.DrawString("Hello APD!", printFont, Brushes.Black, 6, 4)

        ' Indicate that no more data to print, and the Print Document can now send the print data to the spooler.
        e.HasMorePages = False
    End Sub

    ' The event handler function when Timer is started.
    ' This will first lock the printer for exclusive use.
    ' If it can't lock the printer, it means that other program is still accessing the printer and locked it.
    ' If that happens, the program will show the Cancel button so that the user can cancel its printing,
    ' or the program will try to lock the printer from time to time.
    Private Sub tmrTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrTimer.Tick

        ' Lock the printer.
        If m_objAPI.LockPrinter(10) = ErrorCode.SUCCESS Then
            Print()
        Else
            ' If the printer cannot be locked, display a message and the Cancel button.
            lblMessage.Visible = True
            cmdCancel.Visible = True
        End If

    End Sub

    Private Sub Print()

        ' Stop the Timer, disable button and clear display.
        tmrTimer.Stop()
        tmrTimer.Dispose()
        lblMessage.Visible = False
        cmdCancel.Visible = False

        If pdPrint.PrinterSettings.IsValid Then

            pdPrint.DocumentName = "Testing"
            ' Start printing
            pdPrint.Print()

            ' Unlock the printer from the program's exclusive use.
            If m_objAPI.UnlockPrinter = ErrorCode.SUCCESS Then
                ClosePrinter()
            Else
                MessageBox.Show("Failed to unlock printer.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If

        Else
            tmrTimer.Stop()
            MessageBox.Show("Printer is not available.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If

    End Sub

    Private Sub ClosePrinter()

        ' Close Printer Monitor.
        If m_objAPI.IsValid Then
            If Not m_objAPI.CloseMonPrinter = ErrorCode.SUCCESS Then
                MessageBox.Show("Failed to close printer status monitor.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If
        End If

    End Sub

    ' The executed function when the Cancel button is clicked.
    Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click

        ' Stop the Timer, disable button and clear display.
        tmrTimer.Stop()
        lblMessage.Visible = False
        cmdCancel.Visible = False

        MessageBox.Show("Print job cancelled.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Information)

        ClosePrinter()

    End Sub

    ' The executed function when the Close button is clicked.
    Private Sub cmdClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click
        ClosePrinter()
        Close()
    End Sub
End Class
